/*
     This file is part of libextractor.
     (C) 2002, 2003 Vidyut Samanta and Christian Grothoff

     libextractor is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     libextractor is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with libextractor; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 **/


#include "extractor.h"
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

typedef struct {
  char title[31];
  char artist[31];
  char album[31];
  char year[5];
  char comment[31];
} id3tag;

#define DEBUG_EXTRACT_MP3 0

#define OK         0
#define SYSERR     1
#define INVALID_ID3 2

/**
 * Remove trailing whitespace from the end of a string 
 **/
static char * unpad(char * string) {
  char * end = &string[strlen(string) - 1];
  while (' ' == end[0]) 
    (end--)[0]='\0';
  return string;
}

/**
 * Copy a string from src to dst and unpad.
 **/
static void copypad(char * dst,
		    char * src,
		    int len) {
  memcpy(dst, src, len);
  dst[len] = '\0';
  unpad(dst);
}

/**
 * Copy from b to a len characters and advance
 * b by len.
 **/
#define COPYPADADV(a,b,c) copypad(a,b,c); b+=c;

static int get_id3(char * data, size_t size, id3tag * id3) {
  char * pos;

  if (size < 128)
    return INVALID_ID3;

  pos = &data[size - 128];
  if (0 != strncmp((const char *)"TAG",(const char *)pos, 3))
    return INVALID_ID3;      
  pos += 3;
  COPYPADADV(id3->title, pos, 30);
  COPYPADADV(id3->artist, pos, 30); 
  COPYPADADV(id3->album, pos, 30);
  COPYPADADV(id3->year, pos, 4);
  COPYPADADV(id3->comment, pos, 30);
  return OK;
}

struct EXTRACTOR_Keywords * addkword(EXTRACTOR_KeywordList *oldhead, 
				     char *phrase,
				     EXTRACTOR_KeywordType type) {

   EXTRACTOR_KeywordList * keyword;
   
   keyword = (EXTRACTOR_KeywordList*) malloc(sizeof(EXTRACTOR_KeywordList));
   keyword->next = oldhead;    
   keyword->keyword = strdup(phrase);
   keyword->keywordType = type;
   return keyword;
}

/* mimetype = audio/mp3 */
struct EXTRACTOR_Keywords * libextractor_mp3_extract(char * filename,
                                                     char * data,
                                                     size_t size,
                                                     struct EXTRACTOR_Keywords * klist) {
  id3tag info;
  char * word;

  if (0 != get_id3(data, size, &info)) {
#if DEBUG_EXTRACT_MP3
    fprintf(stderr, 
	    "\nIgnoring %s (could not decode id3 header)\n",
	    filename);
#endif
    return klist;
  }

  if (strlen(info.title) > 0) 
    klist = addkword(klist, info.title, EXTRACTOR_TITLE); 
  if (strlen(info.artist) > 0)
    klist = addkword(klist, info.artist, EXTRACTOR_ARTIST);
  if (strlen(info.album) > 0) 
    klist = addkword(klist, info.album, EXTRACTOR_ALBUM);
  if (strlen(info.year) > 0) 
    klist = addkword(klist, info.year, EXTRACTOR_DATE);


  /* A keyword that has all of the information together) */
  word = (char*) malloc(strlen(info.artist) + strlen(info.title) + strlen(info.album) + 6);
  sprintf(word, 
	  "%s: %s (%s)",
	  info.artist, 
	  info.title, 
	  info.album);
  klist = addkword(klist, word, EXTRACTOR_DESCRIPTION);
  free(word);


  /* We would like to do this, but
     we would need logic to make sure that its actually an mp3...  */
  /*  klist = addkword(klist, "audio/mp3", EXTRACTOR_MIMETYPE); */
  
  return klist;
}

/* end of mp3extractor.c */
